# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Create the file components/<name>.ic_curation_summary.RDS for older projects
BackFillIcCurationSummaryClp=function(yamlpath=NULL) {
    pipeline_mgr = PipelineManager$new(
        subdirectory_names=workflow_directories,
        yamlpath=yamlpath,
        stages=all_stages,
        defaults=default_settings_no_history)
    pipeline.parameters = pipeline_mgr$pipeline.parameters
    # Get most recent set of curation files
    curation_history_directories = list.files(path=pipeline_mgr$get_history_directory(), pattern='curation_history[.].*', recursive=FALSE, full.names=TRUE)
    if (length(curation_history_directories) == 0) {
        stop(paste0("No curation history directories found in ", pipeline_mgr$get_history_directory()))
    }

    # Select only those with IC curation curation_sheets
    curation_history_directories = curation_history_directories[sapply(curation_history_directories, contains_ic_curation_sheets)]

    if (length(curation_history_directories) == 0) {
        stop(paste0("No curation history directories with IC curation sheets found in ", pipeline_mgr$get_history_directory()))
    }
    timestamps = file.mtime(curation_history_directories)
    latest_curation_history_directory = curation_history_directories[order(timestamps, decreasing=TRUE)][1]

    curation.files=get_curation_sheet_files(latest_curation_history_directory)

    curation_sheets = lapply(names(curation.files), function(cluster_name) {
        df = read.table(file.path(latest_curation_history_directory, curation.files[cluster_name]), stringsAsFactors=F, sep=",", header=T, quote="\"")
        df$cluster = cluster_name
        return(df)
    })
    saveRDS(merge_curation_sheets(curation_sheets),
        file=file.path(pipeline_mgr$directories$components, paste0(pipeline.parameters$name, ic_curation_summary_extension)))
}

contains_ic_curation_sheets=function(curation_history_directory) {
    return(length(get_curation_sheet_files(curation_history_directory)) > 0)
}
