# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

louvain_subcluster_method="LOUVAIN"
gmm_subcluster_method="GMM"
subcluster_methods=c(louvain_subcluster_method, gmm_subcluster_method)

default_subcluster_louvain_nn=20
default_small_cluster_size=20

current_apply_ic_curation_version=3

# Values used if the corresponding key in yaml is not present.
default_settings=list(
    load_dge=list(
        seurat_compatibility=FALSE,
        MergeDgeHeapSize="40g",
        min_genes=401,
        min_cells=11,
        min_transcripts=1
    ),
    ica=list(
        n.components=60,
        ica.method='C',
        ica.verbose=FALSE
    ),
    cluster=list(
        small.cluster.size=default_small_cluster_size,
        res.clust=0.1,
        skip.first.round=FALSE,
        save.edges=FALSE
    ),
    subica=list(
        min.subclust=200,
        drop.small.cells=0,
        ica.method='C',
        ica.verbose=FALSE
    ),
    subcluster=list(
        center.KDE=TRUE,
        cluster.method=louvain_subcluster_method,
        gmm.model.threshold=0,
        force.resubcluster=FALSE,
        save.edges=FALSE
    ),
    workflow=list(
        random.seed=1,
        stack_trace=TRUE,
        record_history=TRUE,
        warn_level=1,
        clean_intermediates=FALSE
    ),
    differential_expression=list(
        pval.threshold=diffexp_default_pval.thresh,
        FOLDch.threshold=diffexp_default_FOLDch.thresh
    )
)

default_settings_no_history = default_settings
default_settings_no_history$workflow$record_history = FALSE

curation_sheets_dir="curation_sheets"
blank_curation_sheets_dir="blank_curation_sheets"
components_dir="components"
assign_dir="assign"
dge_dir="dge"
tsne_dir="tSNE"
featurplots_dir="featureplots"
canonical_plots_dir="canonical"
dropworks_dir="DropWorks"

workflow_directories=list("postClusteringCellCleaning",
                            tsne_dir,
                            assign_dir,
                            "metacells",
                            "phylo",
                            "featureplots",
                            components_dir,
                            dge_dir,
                            curation_sheets_dir,
                            featurplots_dir,
                            blank_curation_sheets_dir,
                            "cluster_sheets",
                            "blank_cluster_sheets",
                            "phylo",
                            "diffexp"
                            )

all_stages=c("load_dge", "select_genes", "ica", "apply_round1_ic_curation", "cluster", "subica", "agg_subica",
"apply_ic_curation", "subcluster", "cluster_annotation", "differential_expression")

cluster_tSNExy_extension="tSNExy.RDS"
louvain_parameters_sheet_extension=".louvain_params.csv"

gmm_idents_label='idents'
louvain_idents_label='louvain'

pairwise_markers_extension="_pairwiseMarkers.RDS"
cluster_markers_extension="_clusterMarkers.RDS"
firstround_label="_FirstRound"

firstround_clean_pairwise_markers_extension=".FirstRound_cleanPairwiseMarkers.RDS"
firstround_clean_cluster_markers_extension=".FirstRound_cleanClusterMarkers.RDS"
firstround_clean_metacells_extension=".cleanMetacells.RDS"
firstround_manual_ica_extension=".ManualFirstRoundICA.RDS"

curated_tsne_extension=".CURATEDtSNE.RDS"

cell_cluster_outcomes_extension=".cell_cluster_outcomes.RDS"
curation_cell_holdouts_extension=".curation_cell_holdouts.RDS"
first_round_curation_sheet_extension=".FirstRoundAnnotationSheet.csv"

auto_tsne_xy_extension='.Auto_tSNExy.RDS'
manual_tsne_xy_extension='.Manual_tSNExy.RDS'
tsne_xy_extension="_tSNExy.RDS"

ic_curation_summary_extension=".ic_curation_summary.RDS"

cluster_assign_extension=".cluster.assign.RDS"

all_ics_extension=".ICA.RDS"

louvain_edges_extension=".edges"