# MIT License
# 
# Copyright 2018 Broad Institute
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Functions for dealing with older ICA projects that don't have the current project layout

read_agg_subica<-function(pipeline_mgr) {
    pipeline.parameters = pipeline_mgr$pipeline.parameters
    agg_sub_ica_file = paste0(pipeline.parameters$name, all_ics_extension)
    subcluster_file = paste0(pipeline.parameters$name, ".subclusters.RDS")
    if (file.exists(agg_sub_ica_file)) {
        return(readRDS(agg_sub_ica_file))
    } else if (file.exists(subcluster_file)) {
        subclusters = readRDS(subcluster_file)
        return(lapply(subclusters, function(sc) list(gene_loadings=sc$ICAOBJ$A, cell_rotations=sc$ICAOBJ$S, name="ICA", tsne=sc$TSNE)))
    } else {
        # Read each ICA file and tSNE file separately
        stage_mgr = pipeline_mgr$stage_mgr
        # load list of clusters into to.subcluster
        stage_mgr$loadStage("agg_subica", forceLoad=TRUE)
        names(to.subcluster) = to.subcluster
        return(lapply(to.subcluster, function(cluster.num) {
            ica_file = getLatestIcaRds(cluster.num=cluster.num, name=pipeline.parameters$name)
            ica = readRDS(ica_file)
            ica_base = removeSuffix(basename(ica_file), "ICA.RDS")
            tsne_file = file.path(paste0("cluster", cluster.num), paste0(ica_base, "tSNExy.RDS"))
            ica$tsne = readRDS(tsne_file)
            return(ica)
        }))
    }
}

read_round1_ica<-function(pipeline_mgr) {
    pipeline.parameters = pipeline_mgr$pipeline.parameters
    ica_file = file.path(pipeline_mgr$directories$components, paste0(pipeline.parameters$name, ".", ica_file_extension))
    if (file.exists(ica_file)) {
        return(readRDS(ica_file))
    } else {
        old_ica_file = paste0(removeSuffix(ica_file, ica_file_extension), "ica.RDS")
        ica = readRDS(old_ica_file)
        ica$tsne = readRDS(file.path(pipeline_mgr$directories$tSNE, paste0(pipeline.parameters$name, tsne_xy_extension)))
        return(ica)
    }
}