# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

MakeCleanMarkersAndMetacellsClp=function(yamlpath=NULL, ...) {
    pipeline_mgr = PipelineManager$new(
        subdirectory_names=workflow_directories,
        yamlpath=yamlpath,
        stages=NA,
        defaults=default_settings_no_history)
    cell_cluster_outcomes = readRDS(file.path(pipeline_mgr$directories$assign, paste0(pipeline_mgr$pipeline.parameters$name, cell_cluster_outcomes_extension)))
    idents = readRDS(file.path(pipeline_mgr$directories$assign, paste0(pipeline_mgr$pipeline.parameters$name, cluster_assign_extension)))
    raw_dge = loadSparseDge(get_raw_dge_path(pipeline_mgr))
    filtered_raw_dge = readRDS(get_filtered_raw_dge_path(pipeline_mgr))
    filtered_scaled_dge = readRDS(get_filtered_scaled_dge_path(pipeline_mgr))
    clean_markers_and_metacells = MakeCleanMarkersAndMetacells(
        cell_cluster_outcomes=cell_cluster_outcomes,
        idents=idents,
        raw_dge=raw_dge,
        filtered_raw_dge=filtered_raw_dge,
        filtered_scaled_dge=filtered_scaled_dge,
        ...)
    SaveCleanMarkersAndMetacells(pipeline_mgr, clean_markers_and_metacells)
}

SaveCleanMarkersAndMetacells=function(pipeline_mgr, clean_markers_and_metacells) {
    saveRDS(clean_markers_and_metacells$cluster_markers, file=paste0(pipeline_mgr$pipeline.parameters$name, firstround_clean_cluster_markers_extension))
    saveRDS(clean_markers_and_metacells$pairwise_markers, file=paste0(pipeline_mgr$pipeline.parameters$name, firstround_clean_pairwise_markers_extension))
    saveRDS(clean_markers_and_metacells$metacells, file=file.path(pipeline_mgr$directories$metacells, paste0(pipeline_mgr$pipeline.parameters$name, firstround_clean_metacells_extension)))

}

MakeCleanMarkersAndMetacells = function(cell_cluster_outcomes, idents,
        raw_dge, filtered_raw_dge, filtered_scaled_dge,
        FOLDch.thresh=diffexp_default_FOLDch.thresh, pval.thresh=diffexp_default_pval.thresh,
        included_reasons=default_clean_markers_included_reasons) {
    selected_cells = rownames(cell_cluster_outcomes[is.na(cell_cluster_outcomes$reason) |
        cell_cluster_outcomes$reason %in% included_reasons, ])
    selected_idents = idents[selected_cells]
    metacells = sum_dge_by_cluster(idents=selected_idents, raw_dge=raw_dge[, names(selected_idents)])
    pairwise_and_cluster_diffexp = pairwiseAndClusterDiffExpression(idents=selected_idents,
        raw_dge=filtered_raw_dge, scaled_dge=filtered_scaled_dge,
        only.pos=FALSE, FOLDch.thresh=FOLDch.thresh, pval.thresh=FOLDch.thresh)
    return(c(pairwise_and_cluster_diffexp, metacells=list(metacells)))
}
