# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

SelectNumIcsManually = function(cluster.num, n.components) {
    pipeline_mgr = PipelineManager$new(
        subdirectory_names=workflow_directories,
        stages=NA,
        defaults=default_settings_no_history)
    pipeline.parameters = pipeline_mgr$pipeline.parameters
    if (!is.numeric(n.components) || n.components <= 1) {
        stop(paste0("n.components must be > 1, but is ", n.components))
    }

    setwd(normalizePath(pipeline.parameters$workflow$working_directory))

    random.seed=pipeline.parameters$workflow$random.seed
    cluster.dir=get_cluster_directory(cluster.num)
    if (!file.exists(cluster.dir)) {
        stop(sprintf('%s appears to be an invalid cluster number.  Directory %s does not exist', cluster.num, cluster.dir))
    }
    subcluster.inputs = readRDS(paste0(cluster.dir, pipeline.parameters$name, ".subcluster_inputs.RDS"))
    cat_with_time(paste0("Creating ", n.components, " ICs for cluster ", cluster.num))

    ica_and_curate(scaled_gene_selected_dge=subcluster.inputs$scaled_gene_selected_dge,
                    raw_dge=subcluster.inputs$raw_dge,
                    n.components=n.components,
                    cluster.dir=cluster.dir,
                    components.dir=components_dir,
                    blank.curation.sheets.dir=blank_curation_sheets_dir,
                    cluster.num=cluster.num,
                    file_prefix=make_subcluster_file_prefix(project.name=pipeline.parameters$name,
                                                cluster.num=cluster.num,
                                                n.components=n.components,
                                                used_jackstraw=FALSE),
                    random.seed=random.seed,
                    ica.method=pipeline.parameters$subica$ica.method,
                    ica.verbose=pipeline.parameters$subica$ica.verbose)
}
