# MIT License
# 
# Copyright 2018 Broad Institute
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

make_cluster_titrate_path=function(output_dir, samplename, nn, resolution) {
    samplename_and_params = paste0(samplename,".nn", nn, ".resolution", resolution)
    return(file.path(output_dir, paste0(samplename_and_params, cluster_assign_extension)))
}


TitrateCluster<-function(resolutions) {
    pipeline_mgr = PipelineManager$new(
        subdirectory_names=workflow_directories,
        stages=all_stages,
        defaults=default_settings_no_history)

    pipeline.parameters = pipeline_mgr$pipeline.parameters
    pipeline_mgr$stage_mgr$loadStage("cluster", forceLoad=TRUE)

    small.cluster.size = pipeline.parameters$cluster$small.cluster.size
    nn=25
    for (res.clust in resolutions) {
        cat_with_time(sprintf("Clustering with resolution %f", res.clust))
        idents = louvainCluster(data=ica.obj$S,min.clust=small.cluster.size, nn=nn, nstart=100, iter.max=20,
                    jaccard=T,R=res.clust, orig.scale = scaled_centered_gene_selected_dge,
                    SNN=NULL,generate.tree=T,reorder.tree=T,reorder.numeric=T, random.seed=pipeline.parameters$workflow$random.seed)

        saveRDS(idents$louvain, file=
            make_cluster_titrate_path(pipeline_mgr$directories$assign, pipeline.parameters$name, nn, res.clust))
    }
    cat_with_time("Succeeded")
}