# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

TitrateSubcluster=function(nns, resolutions, clusters=NULL) {
    pipeline_mgr = PipelineManager$new(
        subdirectory_names=workflow_directories,
        stages=all_stages,
        defaults=default_settings_no_history)

    pipeline.parameters = pipeline_mgr$pipeline.parameters
    subcluster_parameters = pipeline.parameters$subcluster
    cluster.method = subcluster_parameters$cluster.method
    cluster_method_label = get_or_else(subcluster_parameters$cluster_method_label, cluster.method)
    if (cluster.method != louvain_subcluster_method) {
        stop(sprintf("Cannot titrate subcluster parameters because subcluster: cluster.method == ", cluster.method))
    }
    filtered_scaled_dge = readRDS(get_filtered_scaled_dge_path(pipeline_mgr))
    raw_dge = loadSparseDge(get_raw_dge_path(pipeline_mgr))

    pipeline_mgr$stage_mgr$loadStage("subcluster", forceLoad=TRUE)
    if (!exists("apply_ic_curation_version") || apply_ic_curation_version != current_apply_ic_curation_version) {
        stop("Old version of apply_ic_curation stage file.  Run stage apply_ic_curation again.")
    }
    non.null.ic.objs = curated.ic.objs[sapply(curated.ic.objs, function(v) !is.null(v))]
    if (is.null(clusters)) {
        clusters = names(non.null.ic.objs)
    } else {
        clusters = as.character(clusters)
    }

    for(cluster.name in clusters) {
        ic = curated.ic.objs[[cluster.name]]

        output_dir = ensure_directory_exists(file.path(paste0("cluster", cluster.name), cluster_method_label))
        samplename=paste0(pipeline.parameters$name, ".cluster", cluster.name)
        os=scale(t(filtered_scaled_dge[rownames(ic$A), rownames(ic$S)]), center=TRUE, scale=TRUE)
        idx=which(is.finite(colSums(os)))
        os=os[,idx]
        if (nrow(ic$S) == 1) {
            cat_with_time(sprintf("Not doing subcluster titration for cluster %s because only 1 IC", cluster.name))
        } else {
            lapply(nns, function(nn) {
                edge = makeLouvainEdgeGraph(data=ic$S, nn=nn)
                lapply(resolutions, function(resolution) {
                    output_paths = make_subcluster_titrate_paths(output_dir, samplename, nn, resolution, cluster_method_label)
                    cat_with_time(paste0("Creating ", output_paths$pdf))
                    idents=clusterRound2(data=ic$S, method=louvain_subcluster_method, orig.scale=os,
                            ica.obj=ic, norm=filtered_scaled_dge[,rownames(ic$S)], edge=edge,
                            dge=raw_dge[rownames(filtered_scaled_dge),rownames(ic$S)], resolution=resolution,
                            nn=nn, file=output_paths$pdf,
                            out_edge_file=
                            if (pipeline.parameters$subcluster$save.edges)
                                make_subcluster_edge_path(pipeline.parameters, cluster.name,
                                                                        resolution=resolution, nn=nn)
                            else NA)
                    saveRDS(idents, file=output_paths$rds)
                })})
        }
    }

}

make_subcluster_titrate_paths=function(output_dir, samplename, nn, resolution, cluster_method_label) {
    samplename_and_params = paste0(samplename,".nn", nn, ".resolution", resolution)
    pdf = file.path(output_dir, paste0(samplename_and_params, ".",
    cluster_method_label,".clusterreport.pdf"))
    rds=file.path(output_dir, paste0(samplename_and_params, ".subcluster_titrate.RDS"))
    return(list(pdf=pdf, rds=rds))
}
