# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

##### These functions are no longer used in IcaCluster, but are being saved because they may be useful in future. ####

##### ICA FUNCTIONS #######

icaproject.halves<-function(normalized.dge,n.components) {

    #Randomly select half of the data set for separate ICA.

    cells.1 <- sample(colnames(normalized.dge),size=0.5*ncol(normalized.dge),replace=F)
    cells.2 <- setdiff(colnames(normalized.dge),cells.1)
    data.1 <- scale(t(normalized.dge[,cells.1]),scale=T,center=T)
    data.1[!is.finite(data.1)] <- 0
    data.2 <- scale(t(normalized.dge[,cells.2]),scale=T,center=T)
    data.2[!is.finite(data.2)] <- 0


    ica.1<-run_ica(data.1,n.comp=n.components)
    ica.2<-run_ica(data.2,n.comp=n.components)

    data.2.toproj<-scale(t(normalized.dge[,cells.2]),center=attr(data.1,which = "scaled:center"),scale=attr(data.1,which = "scaled:scale"))
    data.2.toproj[!is.finite(data.2.toproj)]<-0
    data.1.toproj<-scale(t(normalized.dge[,cells.1]),center=attr(data.2,which = "scaled:center"),scale=attr(data.2,which = "scaled:scale"))
    data.1.toproj[!is.finite(data.1.toproj)]<-0

    #Cross-project the data halves onto the other analysis, to get two full datasets in each.
    proj.2on1<-data.2.toproj %*% ica.1$K %*% ica.1$W
    proj.1on2<-data.1.toproj %*% ica.2$K %*% ica.2$W
    all.1<-rbind(ica.1$S,proj.2on1)
    all.2<-rbind(proj.1on2,ica.2$S)

    #Re-arrange the cells so they are in the same order in both analyses, and the same order as the input data.
    rownames(all.1)<-c(cells.1,cells.2)
    rownames(all.2)<-c(cells.1,cells.2)
    all.1<-as.matrix(all.1[colnames(normalized.dge),])
    all.2<-as.matrix(all.2[colnames(normalized.dge),])
    genes.1 <- t(ica.1$A)
    genes.2 <- t(ica.2$A)
    rownames(genes.1) <- rownames(normalized.dge)
    rownames(genes.2) <- rownames(normalized.dge)

    #The returned list:
    toret <- list(CELLS1=all.1,CELLS2=all.2,GENES1=genes.1,GENES2=genes.2,half1=cells.1,half2=cells.2)
    return(toret)
}

### HELPER FUNCTIONS ######

matrix.sort <- function(matrix, require_square=TRUE) {

    if (require_square && nrow(matrix) != ncol(matrix)) stop("Not diagonal")
    if(is.null(rownames(matrix))) rownames(matrix) <- 1:nrow(matrix)

    row.max <- apply(matrix,1,which.max)
    if(all(table(row.max) != 1)) stop("Ties cannot be resolved")

    matrix[names(sort(row.max)),]
}

whiten<-function(X,n.comp) {
    X <- scale(X,scale=F)
    n=nrow(X)
    p=ncol(X)
    X=t(X)
    V <- X %*% t(X)/n
    s <- La.svd(V)
    D <- diag(c(1/sqrt(s$d)))
    K <- D %*% t(s$u)
    K <- matrix(K[1:n.comp, ], nrow=n.comp, ncol=p)
    return(K)
}

pairwisehalves<-function(norm,dge.master,samplename,random.seed=1) {
    set.seed(random.seed)
    cells.1 = sample(colnames(norm),size = ncol(norm)/2)
    cells.2 = setdiff(colnames(norm),cells.1)
    norm.1 = norm[,cells.1]
    norm.2 = norm[,cells.2]
    dge.1 = dge.master[,cells.1]
    dge.2 = dge.master[,cells.2]
    var.genes.1 = mean.var.auto(dge = dge.1,xm = norm.1)
    var.genes.2 = mean.var.auto(dge = dge.2,xm = norm.2)
    print(paste0("Jaccard agreement of var genes: ",round(length(intersect(var.genes.1,var.genes.2))/length(union(var.genes.1,var.genes.2)),3)))

    scale.1 = scale(t(norm.1[var.genes.1,]),center=T,scale=T)
    scale.2 = scale(t(norm.2[var.genes.2,]),center=T,scale=T)
    save(scale.1,file=paste0(samplename,"scaleddata1.Robj"))
    save(scale.2,file=paste0(samplename,"scaleddata2.Robj"))
    ica.obj.1<-run_ica(scale.1, n.components,data.project=NULL)
    ica.obj.2<-run_ica(scale.2, n.components,data.project=NULL)
    attr.1<-attributes(scale.1)
    attr.2<-attributes(scale.2)
    data.proj.1 = as.matrix(sweep(norm.1[var.genes.2,],1,attr.2[[3]],"-"))
    print(length(attr.2[[3]]))
    data.proj.1 = sweep(data.proj.1,1,attr.2[[4]],"/")
    data.proj.2 = as.matrix(sweep(norm.2[var.genes.1,],1,attr.1[[3]],"-"))
    print(length(attr.1[[4]]))
    data.proj.2 = sweep(data.proj.2,1,attr.1[[4]],"/")
    print(dim(data.proj.1))
    print(dim(data.proj.2))
    data.proj.1[!is.finite(data.proj.1)]<-0
    data.proj.2[!is.finite(data.proj.2)]<-0



    proj.2on1<-t(data.proj.2) %*% ica.obj.1$K %*% ica.obj.1$W
    proj.1on2<-t(data.proj.1) %*% ica.obj.2$K %*% ica.obj.2$W
    rots.all.1<-rbind(ica.obj.1$S,proj.2on1)
    rots.all.2<-rbind(proj.1on2,ica.obj.2$S)

    rownames(rots.all.1)<-c(cells.1,cells.2)
    rownames(rots.all.2)<-c(cells.1,cells.2)
    rots.all.1<-rots.all.1[colnames(norm),]
    rots.all.2<-rots.all.2[colnames(norm),]
    ica.obj.1[[1]]<-rots.all.1
    ica.obj.2[[1]]<-rots.all.2
    return(list(ica.obj.1,ica.obj.2))
}

getrrangeclusters <- function(edge,modularity=1,step = diff(r.range)/50,r.range=c(0.1,2),mod.restart = 100,mod.iter=10,cellnames,algorithm = 1) {
    r.range = seq(r.range[1],r.range[2],by=step)
    cat_with_time(r.range)
    data = lapply(r.range,function(r) {

        idents <- modclust(edge=edge, modularity=modularity,resolution=r,n.iter=mod.iter,n.start=10,cellnames=cellnames,algorithm = algorithm)

        N <- length(levels(idents))

        cat_with_time(paste0("Number of clusters detected: ", N))
        return(idents)
    })
    return(data)
}


