% Copyright 2017 Broad Institute
%
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function matlabgmmcluster(dataInFile, cluster0InFile, softOutFile, bicOutFile, logLikOutFile, lambdaOutFile, muOutFile, sigmaOutFile, tolFun)

    pwd
    %% Estimation
    data=csvread(dataInFile);
    cluster0=csvread(cluster0InFile);
    k=max(cluster0)
    converged=0
    options = statset('MaxIter',1000)

    if (exist('tolFun', 'var'))
        disp(['Using tolFun', num2str(tolFun)])
        options = statset(options, 'TolFun', tolFun)
    else
        disp('Using default tolFun')
    end

    %while converged <1
    gmfit=fitgmdist(data,k,'Start',cluster0,'CovType','full','SharedCov',true,'options',options,'Regularize',1e-6)
    converged=gmfit.Converged
    %k=k-1
    %end

    P=posterior(gmfit,data);

    csvwrite(softOutFile,P);
    BIC=gmfit.BIC;
    csvwrite(bicOutFile,BIC);
    loglik=gmfit.NlogL
    csvwrite(logLikOutFile,loglik);

    sigma=gmfit.Sigma(:,:,1);
    mu=gmfit.mu;
    clusters=cluster(gmfit,data);
    q=unique(clusters);
    out=[q,histc(clusters(:),q)];
    lambda=out/sum(out);
    csvwrite(lambdaOutFile,lambda);
    csvwrite(muOutFile,mu);
    csvwrite(sigmaOutFile,sigma);

end