# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

compare_robjs = function(newFile, oldFile, verbose=TRUE, tolerance=0) {
    if (!check_file_existence(newFile) || !check_file_existence(oldFile)) {
        return(FALSE)
    }
    equal = TRUE
    newVals = mget(load(newFile))
    oldVals = mget(load(oldFile))
    if (!identical(names(newVals), names(oldVals))) {
        equal = FALSE
        diffs = c(paste0("names(", newFile, ") !=  names(", oldFile, ")"),
                paste0(paste(names(newVals), collapse=", "), " != ", paste(names(oldVals), collapse=", ")))
    } else {
        diffs = c()
        for (i in 1:length(newVals)) {
            newdiffs = all.equal(newVals[i], oldVals[i], tolerance=tolerance)
            if (is.character(newdiffs)) {
                diffs=append(diffs, newdiffs)
            }
        }
        if (length(diffs) > 0) {
            equal = FALSE
        }
    }
    if (verbose || !equal) {
        if (equal) message = " matched" else message = " do not match"
        print(paste0(newFile, " and ", oldFile, message))
        if (verbose && !equal) {
           print(diffs)
        }
    }
    return(equal)
}

compare_rds = function(newFile, oldFile, verbose=TRUE, tolerance=0) {
    if (!check_file_existence(newFile) || !check_file_existence(oldFile)) {
        return(FALSE)
    }
    newVal = readRDS(newFile)
    oldVal = readRDS(oldFile)
    diffs = all.equal(newVal, oldVal, tolerance=tolerance)
    equal = TRUE
    if (is.character(diffs)) {
        equal = FALSE
    }
    if (verbose || !equal) {
        if (equal) message = " matched" else message = " do not match"
        print(paste0(newFile, " and ", oldFile, message))
        if (verbose && !equal) {
           print(diffs)
        }
    }
    return(equal)
}

check_file_existence = function(f) {
    if (!file.exists(f)) {
        print(paste0(f, " does not exist"))
        return(FALSE)
    } else {
        return(TRUE);
    }
}
