# MIT License
#
# Copyright 2017 Broad Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Get a value (typically from a yaml object), or default_val if val is null
get_or_else = function(val, default_val) {
    if (is.null(val)) {
        return(default_val)
    } else {
        return(val)
    }
}

#' @param a and b are both factors, but it is not known whether the levels of a and the levels of b are equivalent
#' @return list(confusion_matrix of size of intersection between corresponding levels of a and b, as a fraction of the size of b for that level,
#               list of values in a only,
#               list of values in b only)
make_class_intersection_matrix=function(a, b) {
    common_names = intersect(names(a), names(b))
    a_only = setdiff(names(a), common_names)
    b_only = setdiff(names(b), common_names)
    a = a[common_names]
    b = b[common_names]
    ret = matrix(nrow=length(levels(a)), ncol=length(levels(b)), dimnames=list(levels(a), levels(b)))
    for (a_level in levels(a)) {
        this_a = names(a[a==a_level])
        for(b_level in levels(b)) {
            this_b = names(b[b==b_level])
            num_this_b = length(this_b)
            ret[a_level, b_level] = length(intersect(this_a, this_b))/num_this_b
        }
    }
    return(list(confusion_matrix=ret, a_only=a_only, b_only=b_only))
}
